@@ -27,6 +27,10 @@ Free Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA.  */
 
 #include <fcntl.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #ifdef HAVE_STDLIB_H
 #  include <stdlib.h>
 #else
@@ -35,6 +39,14 @@ extern char *malloc ();
 extern char *realloc ();
 #endif
 
+#if defined (HAVE_STRING_H)
+#include <string.h>
+#endif
+
+#if !defined (HAVE_BCOPY) && (defined (HAVE_STRING_H) || defined (STDC_HEADERS))
+#  define bcopy(s, d, n)	memcpy ((d), (s), (n))
+#endif
+
 #else /* not HAVE_CONFIG_H */
 
 #ifdef STDC_HEADERS
